local C = Growler_Constants
local L = BixLib
local O = Growler_Options

Growler_Stable = CreateFrame("Frame","Growler_Stable",UIParent)
local S = Growler_Stable

S.CreateMainUI = function()
	S.Tabs = {}
	S.StableSlots = {}
	S.PetBrowserPages = {}
	S.IsShown = false
	S.lastupdate = 0

	S.ShowStable = function()
		S.UpdateStables()
		S.IsShown = true
		S:Show()
		PlaySoundKitID(829)
		Growler_StableBrowser.show()
	end

	S.HideStable = function()
		S.IsShown = false
		S:Hide()
		PlaySoundKitID(830)
		Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_up)
	end
	
	S:SetWidth((C.stablePadding*2) + ((C.stableSpacing+C.stableslotSize)*6) - C.stableSpacing + C.stablemodelSize)
	S:SetHeight(C.stableTitleHeight + (C.stablePadding*1) + (C.stableslotSize+C.stableSpacing)*10 + (C.stableSpacing*7))
	S:SetPoint("CENTER",World,"CENTER",0,200)
	S:Hide()
	S:SetFrameStrata("HIGH")
	S:SetFrameLevel(2)
	S:SetMovable(false)
	S:SetClampedToScreen(true)
	S:SetScript("OnUpdate",function(self,elapsed)
		self.lastupdate = self.lastupdate + elapsed
	end)
	local backdrop = {
	  bgFile = "Interface\\FrameGeneral\\UI-Background-Rock",
	  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
	  tile = false,
	  tileSize = 50,
	  edgeSize = 32,
	  insets = {
		left = 5,
		right = 5,
		top = 5,
		bottom = 5
	  }
	}
	S:SetBackdrop(backdrop)
	S:SetBackdropColor(1,0.8,0.3,1)
	S:SetBackdropBorderColor(1,1,1,1)
	
	S.CreateTitleFrame()
	S.CreateStableBrowser()
	--S.CreatePetBrowser()
	local mframe = S.CreateModelFrame("ModelFrame",Growler_StableBrowser,true)
	S.CreatePetSlots()
	S.SelectPetSlot()
	--S.ShowStable()
end

S.CreateTitleFrame = function()
	local tf = CreateFrame("Button","GrowlerStable_TitleFrame",S)
	tf:SetSize(S:GetWidth(),C.stableTitleHeight)
	tf:SetPoint("TOP",S,"TOP",0,0)
	tf:SetFrameLevel(4)
	local backdrop = {
	  bgFile = "Interface\\FrameGeneral\\UI-Background-Rock",
	  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
	  tile = false,
	  tileSize = 50,
	  edgeSize = 32,
	  insets = {
		left = 5,
		right = 5,
		top = 5,
		bottom = 5
	  }
	}
	tf:SetBackdrop(backdrop)
	tf:SetBackdropColor(1,0.8,0.3,1)
	tf:SetBackdropBorderColor(1,1,1,1)
	
	local text = tf:CreateFontString("GrowlerStable_TitleFrame_Text", "OVERLAY", "GameFontNormalHuge")
	text:SetPoint("CENTER",tf,"CENTER",0,0)
	text:SetTextHeight(20)
	text:SetText("Stables")
	tf.text = text
	
	tf:RegisterForDrag("LeftButton")
	tf:SetScript("OnDragStart",function(self)
		S:SetMovable(true)
		S:StartMoving()
	end)
	tf:SetScript("OnDragStop",function(self)
		S:StopMovingOrSizing()
		S:SetMovable(false)
	end)
	
	local exitbtn = CreateFrame("Button","GrowlerStable_Exitbtn",tf)
	exitbtn:SetSize(30,30)
	exitbtn:SetPoint("RIGHT",tf,"RIGHT", -15,0)
	exitbtn.texture_up = "Interface\\BUTTONS\\UI-Panel-MinimizeButton-Up"
	exitbtn.texture_down = "Interface\\BUTTONS\\UI-Panel-MinimizeButton-Down"
	L.SetTexture(exitbtn.texture_up,exitbtn)
	exitbtn.texture:SetPoint("TOPLEFT",exitbtn,"TOPLEFT",-10,10)
	exitbtn.texture:SetPoint("BOTTOMRIGHT",exitbtn,"BOTTOMRIGHT",10,-10)
	exitbtn:RegisterForClicks("LeftButtonDown","LeftButtonUp")
	exitbtn:SetScript("OnClick",function(self,button,down)
		if down then
			self.texture:SetTexture(self.texture_down)
		else
			self.texture:SetTexture(self.texture_up)
			S.HideStable()
		end
	end)
	exitbtn:SetScript("OnMouseUp",function(self,button)
		
	end)
end

S.CreateTab = function(frame,icon)
	local tab = CreateFrame("Button",frame:GetName() .. "_Tab",S)
	frame.tab = tab
	tab:SetSize(C.stabletabWidth,C.stabletabHeight)
	tab:SetPoint("TOPLEFT",S,"TOPRIGHT",-15,-L.tablesize(S.Tabs)*(C.stabletabHeight-15))
	S.Tabs[frame:GetName()] = frame
	tab:SetFrameLevel(0)
	
	tab.Select = function()
		frame.tab:SetFrameLevel(1)
		PlaySoundKitID(829)
		tab:SetSize(C.stabletabWidth,C.stabletabHeight)
		local backdrop = {
		  bgFile = "Interface\\ICONS\\" .. icon,
		  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		  tile = false,
		  tileSize = 50,
		  edgeSize = 32,
		  insets = {
			left = 5,
			right = 5,
			top = 5,
			bottom = 5
		  }
		}
		tab:SetBackdrop(backdrop)
	end
	tab.UnSelect = function()
		frame.tab:SetFrameLevel(0)
		tab:SetSize(C.stabletabWidth-5,C.stabletabHeight-5)
		local backdrop = {
		  bgFile = "Interface\\ICONS\\" .. icon,
		  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		  tile = false,
		  tileSize = 50,
		  edgeSize = 25,
		  insets = {
			left = 5,
			right = 5,
			top = 5,
			bottom = 5
		  }
		}
		tab:SetBackdrop(backdrop)
	end
	tab.UnSelect()
	
	tab:SetScript("OnClick",function(self,button)
		if button == "LeftButton" then
			for n,f in pairs(S.Tabs) do
				if f == frame then
					f.tab.Select()
					f.show()
				else
					f.tab.UnSelect()
					f.hide()
				end
			end
		end
	end)
end

S.CreateStableBrowser = function()
	local sb = CreateFrame("Frame","Growler_StableBrowser",S)
	sb:ClearAllPoints()
	sb:SetAllPoints(S)
	sb:SetFrameLevel(2)
	
	sb.IsShown = false
	sb.show = function()
		sb.IsShown = true
		sb:Show()
		PlaySoundKitID(829)
	end
	sb.hide = function()
		sb.IsShown = false
		sb:Hide()
	end
	
	-- local text = sb:CreateFontString("GrowlerStable_ComingSoon_Text", "OVERLAY", "GameFontNormalHuge")
	-- text:SetPoint("BOTTOMRIGHT",sb,"BOTTOMRIGHT",-30,250)
	-- text:SetTextHeight(18)
	-- text:SetText("|cff990000More features coming soon, \nsuch as info about pet name, \nfamily, abilities, spec etc.")
	-- sb.text = text
	
	--S.CreateTab(sb,"ability_mount_ridinghorse")
	--sb.tab.Select()
end

S.GetSelectedPet = function()
	for i=1,55 do
		if S.StableSlots[i].selected then return S.StableSlots[i] end
	end
end
S.SelectPetSlot = function()
	local hasPet, isHunterPet = HasPetUI()
	if hasPet and isHunterPet then
		local curName,_ = UnitName("pet")
		local curFamily = UnitCreatureFamily("pet")
		local curIcon = GetPetIcon()
		for i=1,5 do
			local PI, PN, _, PT, _ = GetStablePetInfo(i)
			if PI ~= nil and PI == curIcon and PN == curName and PT == curFamily then
				if S.GetSelectedPet() ~= S.StableSlots[i] then
					S.StableSlots[i].Select()
				end
			end
		end
	end
end

S.CreatePetSlots = function()
	for i=1,55 do
		local icon, name, level, family, talent = i<=5 and GetStablePetInfo(i) or S.GetStablePet(i)

		local s = getglobal("GrowlerStable_Slot" .. i) or CreateFrame("Button","GrowlerStable_Slot" .. i,Growler_StableBrowser)
		S.StableSlots[i] = s
		s.slot = i
		s:SetSize(C.stableslotSize,C.stableslotSize)
		s:SetFrameLevel(3)
		local pos_x = 0
		local pos_y = 0
		if i <= 5 then
			pos_x = C.stablePadding*2 - C.stableSpacing + ((C.stableSpacing + C.stableslotSize) * 5)
			pos_y = -C.stableTitleHeight -(C.stablePadding*1) - ((C.stableSpacing + C.stableslotSize) * (i-1))
		else
			pos_x = C.stablePadding + ((C.stableSpacing + C.stableslotSize) * ((i-1) % 5))
			pos_y = -C.stableTitleHeight - ((C.stableSpacing + C.stableslotSize) * (math.floor((i-6)/5))) - (C.stableSpacing * 2 * (math.floor((i-6)/10)))
		end
		s:SetPoint("TOPLEFT",Growler_StableBrowser,"TOPLEFT",pos_x,pos_y)
		
		if icon == "" then
			icon = "Interface\\BUTTONS\\UI-EmptySlot"
			L.SetTexture(icon,s)
			s.texture:SetVertexColor(1,0.8,0.3,1)
			s.texture:ClearAllPoints()
			s.texture:SetPoint("TOPLEFT",s,"TOPLEFT",-10,10)
			s.texture:SetPoint("BOTTOMRIGHT",s,"BOTTOMRIGHT",10,-10)
			s.hasPet = false
		else
			local backdrop = {
			  edgeFile = "Interface\\FriendsFrame\\PendingFriendGlow",
			  tile = false,
			  tileSize = 50,
			  edgeSize = 5,
			  insets = {
				left = 5,
				right = 5,
				top = 5,
				bottom = 5
			  }
			}
			s:SetBackdrop(backdrop)
			s:SetBackdropBorderColor(1,1,1,1)
			L.SetTexture(icon,s)
			s.texture:SetVertexColor(1,1,1,1)
			s.hasPet = true
		end
		
		s.selected = false
		function s.Select()
			if s.hasPet then
				for n=1,55 do
					S.StableSlots[n]:Deselect()
				end
				s.selected = true
				local backdrop = {
				  edgeFile = "Interface\\FriendsFrame\\PendingFriendGlow",
				  tile = false,
				  tileSize = 50,
				  edgeSize = 5,
				  insets = {
					left = 5,
					right = 5,
					top = 5,
					bottom = 5
				  }
				}
				s:SetBackdrop(backdrop)
				s:SetBackdropBorderColor(0.5,1,0.7,1)
				SetPetStablePaperdoll(GrowlerStable_ModelFrame,s.slot)
			end
		end
		function s.Deselect()
			if s.hasPet then
				s.selected = false
				local backdrop = {
				  edgeFile = "Interface\\FriendsFrame\\PendingFriendGlow",
				  tile = false,
				  tileSize = 50,
				  edgeSize = 5,
				  insets = {
					left = 5,
					right = 5,
					top = 5,
					bottom = 5
				  }
				}
				s:SetBackdrop(backdrop)
				s:SetBackdropBorderColor(0.6,0.6,0.6,1)
			end
		end
		s:SetScript("OnClick",function(self,button)
			s.Select()
		end)
	end
	
	S.SelectPetSlot()
end

-- S.CreatePetBrowserPage = function(family,sortednames)
	-- local page = CreateFrame("Frame","Growler_PetBrowserPage_" .. family,Growler_PetBrowser_Pages)
	-- page.modelframes = {}
	-- Growler_PetBrowser.pages[family] = page
	-- S.PetBrowserPages[family] = page
	-- page:ClearAllPoints()
	-- page:SetAllPoints(S)
	-- page:Hide()
	
	-- local modelview
	-- local subcat = 0
	-- local modelnum = 1
	-- if L.pets.models[family] then
		-- for ctype,colors in pairs(L.pets.models[family]) do
			-- ctype = L.replace(L.replace(L.replace(ctype," ",""),"/",""),"-","")
			-- local colorNum = 1
			-- for color,models in pairs(colors) do
				-- if modelnum == 1 then modelview = models[1] end
				-- color = L.replace(L.replace(L.replace(color," ",""),"/",""),"-","")
				-- local mframe = S.CreateModelFrame("ModelFrame_" .. family .. "_" .. ctype .. "_" .. color,page,false,models[1])
				-- mframe.page = page
				-- table.insert(page.modelframes,mframe)
				
				-- local cols = C.stablebrowserCols
				-- local x = C.stablePadding + ((C.stableSpacing + C.stablebrowsermodelSize) * ((colorNum-1) % cols))
				-- local y = -C.stablePadding - ((C.stableSpacing + C.stablebrowsermodelSize) * (math.floor((colorNum-1)/cols)+1)) - ((C.stableSpacing + C.stablebrowsermodelSize) * (math.floor((modelnum-colorNum-1)/cols)))
				-- mframe:SetPoint("TOPLEFT",page,"TOPLEFT",x,y)
				-- modelnum = modelnum + 1
				-- colorNum = colorNum + 1
			-- end
			-- subcat = subcat + 1
		-- end
		
		-- local mframe = S.CreateModelFrame("ModelFrame_" .. family,page,true,modelview)
		-- mframe:SetPoint("TOPRIGHT",page,"TOPRIGHT",-C.stablePadding,-C.stablePadding)
		-- page.mframe = mframe
	-- end
	
	
	-- page.IsShown = false
	-- page.show = function()
		-- page.IsShown = true
		-- page:Show()
		-- PlaySoundKitID(829)
		-- page.modelframes[1].Select()
	-- end
	-- page.hide = function()
		-- page.IsShown = false
		-- page:Hide()
	-- end
	
	-- return page
-- end

-- S.CreatePetBrowserCategoryIcons = function()
	-- local i = 1
	-- local name = {}
	-- for n,p in pairs(L.pets.families) do
		-- table.insert(name,n)
		-- i = i + 1
	-- end
	-- table.sort(name)
	
	-- for i=1,table.getn(name) do
		-- local fam = CreateFrame("Button","Growler_PetBrowserCategory_" .. name[i],Growler_PetBrowser_Categories)
		-- fam:SetSize(C.stablecatSize,C.stablecatSize)
		-- local cols = 12
		-- local x = C.stablePadding + ((C.stableSpacing + C.stablecatSize) * ((i-1) % cols))
		-- local y = -C.stablePadding - ((C.stableSpacing + C.stablecatSize) * (math.floor((i-1)/cols))) -- (C.stableSpacing * 2 * (math.floor((i-1)/10)))
		-- fam:SetPoint("TOPLEFT",Growler_PetBrowser,"TOPLEFT",x,y)
		-- local path = "Interface\\ICONS\\" .. L.pets.families[name[i]].icon
		-- L.SetTexture(path,fam)
		
		-- local backdrop = {
		  -- edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
		  -- tile = false,
		  -- tileSize = 32,
		  -- edgeSize = 10,
		  -- insets = {5,5,5,5}
		-- }
		-- fam:SetBackdrop(backdrop)
		-- fam:SetBackdropBorderColor(1,1,1,1)
		
		-- fam.page = S.CreatePetBrowserPage(name[i],name)
		
		-- fam:SetScript("OnClick",function(self,button)
			-- Growler_PetBrowser_Categories.hide()
			-- Growler_PetBrowser_Pages.show()
			-- fam.page.show()
		-- end)
		
		-- i = i + 1
	-- end
-- end


-- S.CreatePetBrowser = function()
	-- local pb = CreateFrame("Frame","Growler_PetBrowser",S)
	-- pb.pages = {}
	-- pb:ClearAllPoints()
	-- pb:SetAllPoints(S)
	-- local cats = CreateFrame("Frame","Growler_PetBrowser_Categories",pb)
	-- cats:ClearAllPoints()
	-- cats:SetAllPoints(pb)
	-- local pages = CreateFrame("Frame","Growler_PetBrowser_Pages",pb)
	-- pages:ClearAllPoints()
	-- pages:SetAllPoints(pb)
	
	-- S.CreatePetBrowserCategoryIcons()
	
	-- cats.IsShown = false
	-- pages.IsShown = false
	-- pb.IsShown = false
	-- cats.show = function()
		-- cats.IsShown = true
		-- cats:Show()
		-- pages.hide()
	-- end
	-- cats.hide = function()
		-- cats.IsShown = false
		-- cats:Hide()
	-- end
	
	-- pages.show = function()
		-- pages.IsShown = true
		-- pages:Show()
		-- cats.hide()
	-- end
	-- pages.hide = function()
		-- pages.IsShown = false
		-- pages:Hide()
		-- for name in pairs(L.pets.families) do
			-- S.PetBrowserPages[name].hide()
		-- end
	-- end
	
	-- pb.categoryPage = cats	
	-- S.CreateTab(pb,"inv_helm_cloth_petsafari_a_01")
	
	-- pb.show = function()
		-- pb.IsShown = true
		-- pb:Show()
		-- cats.show()
		-- pages.hide()
	-- end
	-- pb.hide = function()
		-- pb.IsShown = false
		-- pb:Hide()
		-- cats.hide()
		-- pages.hide()
	-- end
	-- cats.hide()
	-- pages.hide()
	-- pb.hide()
	
	-- pb.DeselectAll = function()
		-- for n,p in pairs(pb.pages) do
			-- for nn,pp in pairs(p.modelframes) do
				-- pp.Deselect()
			-- end
		-- end
	-- end
-- end

S.CreateModelFrame = function(name,parentFrame,IsBig,creatureID)
	local mframe = CreateFrame("DressUpModel","GrowlerStable_" .. name,parentFrame)
	
	local bgF = "Interface\\FrameGeneral\\UI-Background-Rock"
	local edgeF = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border"
	local edgeS = 18
	local size = C.stablebrowsermodelSize
	if IsBig then
		bgF = "Interface\\DRESSUPFRAME\\DressUpBackground-Goblin1"
		edgeF = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border"
		edgeS = 32
		size = C.stablemodelSize
	else
	end
	local backdrop = {
	  bgFile = bgF,
	  edgeFile = edgeF,
	  tile = false,
	  tileSize = 32,
	  edgeSize = edgeS,
	  insets = {left = 5,right = 5,top = 5,bottom = 5}
	}
	mframe:SetSize(size,size)
	mframe:SetBackdrop(backdrop)
	if creatureID == nil then
		SetPetStablePaperdoll(mframe,1)
	else
		mframe:SetCreature(creatureID)
	end
	if IsBig then
		mframe:SetPoint("TOPRIGHT",Growler_StableBrowser,"TOPRIGHT",-C.stablePadding,-C.stableTitleHeight)
		mframe:SetFrameLevel(4)
		mframe:SetBackdropColor(1,1,1,1)
		mframe:SetBackdropBorderColor(1,1,1,1)
		mframe:EnableMouse(true)
		mframe:EnableMouseWheel(true)
		mframe.lastx, mframe.lasty = 0,0
		mframe.lastupdate = 0
		mframe.IsRotating = false
		mframe:RegisterForDrag("LeftButton","RightButton")
		mframe:SetScript("OnDragStart",function(self,button)
			mframe.lastx, mframe.lasty = GetCursorPosition()
			if button == "LeftButton" then
				mframe.IsRotating = true
			end
		end)
		mframe:SetScript("OnDragStop",function(self)
			mframe.IsRotating = false
		end)
		mframe:SetScript("OnUpdate",function(self,elapsed)
			mframe.lastupdate = mframe.lastupdate + elapsed
			if mframe.lastupdate > 0.01 then
				mframe.lastupdate = 0
				if mframe.IsRotating then
					local newx, newy = GetCursorPosition()
					local mult = newx/mframe.lastx
					mframe:SetFacing(mframe:GetFacing() + (math.pi*mult-math.pi))
					mframe.lastx = newx
				end
			end
		end)
		mframe:SetScript("OnMouseWheel",function(self,dir)
			local mscale = self:GetModelScale()
			if (dir == -1 and mscale > 0.3) or (dir == 1 and mscale < 2) then
				self:SetModelScale(mscale+(0.1*dir))
			end
		end)
	else
		mframe.selected = false
		mframe:SetBackdropColor(0.5,0.4,0.1,1)
		mframe:SetBackdropBorderColor(0.7,0.7,0.7,1)
		mframe.Select = function()
			Growler_PetBrowser.DeselectAll()
			mframe.selected = true
			mframe:SetBackdropColor(0.5,0.7,0.1,1)
			mframe:SetBackdropBorderColor(0.7,1,0.7,1)
			mframe.page.mframe:SetCreature(creatureID)
		end
		mframe.Deselect = function()
			mframe.selected = false
			mframe:SetBackdropColor(0.5,0.4,0.1,1)
			mframe:SetBackdropBorderColor(0.7,0.7,0.7,1)
		end
	end
	
	mframe:SetFacing(0.5)

	mframe:SetScript("OnMouseUp",function(self,button)
		if not IsBig then
			self.Select()
		end
	end)
	mframe:Show()
	return mframe
end

S.GetStablePet = function(i)
	return GrowlerDB.pets[i].icon, GrowlerDB.pets[i].name, GrowlerDB.pets[i].level, GrowlerDB.pets[i].family, GrowlerDB.pets[i].talent
end
S.UpdateStables = function()
	for i=1,55 do
		GrowlerDB.pets[i].icon = select(1,GetStablePetInfo(i)) or ""
		GrowlerDB.pets[i].name = select(2,GetStablePetInfo(i)) or ""
		GrowlerDB.pets[i].level = select(3,GetStablePetInfo(i)) or ""
		GrowlerDB.pets[i].family = select(4,GetStablePetInfo(i)) or ""
		GrowlerDB.pets[i].talent = select(5,GetStablePetInfo(i)) or ""
	end
	S.CreatePetSlots()
	S.SelectPetSlot()
end